--[[
	ENTITY
	Sprite extention useful for lots of in-game objects represented graphically.
]]

class "Entity" (Sprite) {
	__init__ = function(self, spriteName, posX, posY, hitBox)
		Sprite.__init__(self, spriteName, math.floor(posX or 1), math.floor(posY or 1))

		self.posX = posX or 1
		self.posY = posY or 1
		self.hp = 1
		self.move = {}
		self.velocityX = 0
		self.velocityY = 0
		self.acceleration = 400
		self.brake = 600
		self.maxVelocity = 350
		self.hitBox = hitBox or {x = 0, y = 0, w = self.meta.width, h = self.meta.height}
		local mt = { __call = function() return self.hitBox.x + self.posX, self.hitBox.y + self.posY, self.hitBox.w, self.hitBox.h end }
		setmetatable(self.hitBox, mt)

	end,

	update = function(self, dt, t)
		if self.move.up then
			if self.velocityY > 0 then
				self.velocityY = self.velocityY - self.brake * dt
			else
				self.velocityY = math.max(-self.maxVelocity, self.velocityY - (1.5 * self.acceleration) * dt)
			end
		end
		if self.move.down then
			if self.velocityY < 0 then
				self.velocityY = self.velocityY + self.brake * dt
			else
				self.velocityY = math.min(self.maxVelocity, self.velocityY + (1.5 * self.acceleration) * dt)
			end
		end
		if self.move.left then
			if self.velocityX > 0 then
				self.velocityX = self.velocityX - self.brake * dt
			else
				self.velocityX = math.max(-self.maxVelocity, self.velocityX - self.acceleration * dt)
			end
		end
		if self.move.right then
			if self.velocityX < 0 then
				self.velocityX = self.velocityX + self.brake * dt
			else
				self.velocityX = math.min(self.maxVelocity, self.velocityX + self.acceleration * dt)
			end
		end

		if not self.move.left and not self.move.right then
			if self.velocityX > 0 then
				self.velocityX = self.velocityX > self.brake * dt and self.velocityX - self.brake * dt or 0
			elseif self.velocityX < 0 then
				self.velocityX = math.abs(self.velocityX) > self.brake * dt and self.velocityX + self.brake * dt or 0
			end
		end

		if not self.move.up and not self.move.down then
			if self.velocityY > 0 then
				self.velocityY = self.velocityY > self.brake * dt and self.velocityY - self.brake * dt or 0
			elseif self.velocityY < 0 then
				self.velocityY = math.abs(self.velocityY) > self.brake * dt and self.velocityY + self.brake * dt or 0
			end
		end

		local magnitude = (self.velocityX^2 + self.velocityY^2)^0.5
		magnitude = magnitude == 0 and 1 or magnitude
		local velX = self.velocityX / magnitude + self.velocityX
		local velY = self.velocityY / magnitude + self.velocityY

		local magVel = (self.velocityX^2 + self.velocityY^2)^0.5
		if magVel > self.maxVelocity then
			self.velocityX = self.velocityX * self.maxVelocity / magVel
			self.velocityY = self.velocityY * self.maxVelocity / magVel
		end

		self.posX = self.posX + self.velocityX * dt
		self.posY = self.posY + self.velocityY * dt

		--update sprite
		self.drawX = math.floor(self.posX)
		self.drawY = math.floor(self.posY)
		Sprite.update(self, dt, t)

		--check life
		self.kill = self.hp < 0 and true
	end,

	draw = function(self)
		Sprite.draw(self)
	end,

	collide = function(self, objX, objY, objW, objH)
		local x = self.posX + self.hitBox.x
		local y = self.posY + self.hitBox.y
		if not (objX + objW < x  or  x + self.hitBox.w < objX  or  objY + objH < y  or  y + self.hitBox.h < objY) then
			return true
		end
	end
}