--[[
	GAME
	State manager and main superclasstypeobjectmegahyperthing.
]]

local state = {} --available gamestates

class "Game" {
	__init__ = function(self)
		--create gamestates
		assert(love.filesystem.isDirectory("state"), "No states could be found!")
		local stateFiles = love.filesystem.enumerate("state")
		for k, file in ipairs(stateFiles) do
			local name = file:match("^(.+)%.lua$")
			if name then
				state[name] = Gamestate(name, file)
			end
		end
		
		--probably temp junk
		love.graphics.setDefaultImageFilter("nearest", "nearest")
		
		--setup font object
		self.system = {font = cache.font("font/prstartk.ttf:8")}
		love.graphics.setFont(self.system.font)
		
		self:changeState("init")
	end,	
	
	update = function(self, dt)
		local t = love.timer.getTime()
		self.state:update(dt, t)
	end,
	
	draw = function(self)
		self.state:draw()
	end,
	
	focus = function(self, f)
		self.state:focus(f)
	end,
	
	quit = function(self)
		self.state:quit()
	end,
	
	keypressed = function(self, key, unicode)
		self.state:keypressed(key, unicode)
	end,
	
	keyreleased = function(self, key)
		self.state:keyreleased(key)
	end,
	
	joystickpressed = function(self, joystick, button)
		self.state:joystickpressed(joystick, button)
	end,
	
	joystickreleased = function(self, joystick, button)
		self.state:joystickreleased(joystick, button)
	end,
	
	mousepressed = function(self, x, y, button)
		self.state:mousepressed(x, y, button)
	end,
	
	mousereleased = function(self, x, y, button)
		self.state:mousereleased(x, y, button)
	end,
	
	changeState = function(self, name)
		assert(name, "changeState issued with invalid state name!")
		assert(state[name], "Unknown gamestate " .. name .. "!")
		self.state = state[name]
		self.state:reset(self)
	end,
	

	print = function(self, text, x, y, color)
		color = color or {255, 255, 255, 255}
		love.graphics.setColor(10, 10, 10, 128)
		love.graphics.print(text, x + 1, y + 1)
		love.graphics.setColor(color)
		love.graphics.print(text, x, y)
	end,
	
	file = require("lib.jupiter")
}

class "Gamestate" {
	__init__ = function(self, name, file)
		assert(name, "State declared with no name.")
		assert(not state[name], "State declared with ambiguous name.")
		assert(love.filesystem.exists("state/" .. file), "No state file found.")
		local stateSource = love.filesystem.load("state/" .. file)()
		for k, v in pairs(stateSource) do
			self[k] = v --assimilate source
		end
		
		local callbacks = {"reset", "update", "draw", "focus", "quit", "keypressed", "keyreleased", "joystickpressed", "joystickreleased", "mousepressed", "mousereleased"}
		for k, call in ipairs(callbacks) do
			self[call] = self[call] or function() end --default callbacks for stability
		end	
	end
}