--[[
	PLAYER
	THe player ship.
]]

class "Player" (Entity) {
	__init__ = function(self, spriteName, posX, posY, life, weapon)
		Entity.__init__(self, spriteName, posX or 100, posY or 200, {x = 17, y = 0, w = 20, h = 26})
		self.life = life or 5
		self.inventory = {
			cannon = {
				name = "Vulcan",
				maxAmmo = math.huge,
				time = 0,
				cool = 0,
				fire = function(self, state)
					table.insert(state.playerFire, Bullet("bullet", self.posX + self.meta.width, self.posY + self.meta.height / 2))
				end
			},
			double = {
				name = "Double Shot",
				maxAmmo = 200,
				fireCost = 2,
				time = 0,
				cool = 0,
				fire = function(self, state)
					table.insert(state.playerFire, Bullet("bullet_double", self.posX + self.meta.width, self.posY + self.meta.height / 2 - 4))
					table.insert(state.playerFire, Bullet("bullet_double", self.posX + self.meta.width, self.posY + self.meta.height / 2 + 4))
				end
			},
			rapid = {
				name = "Rapid Fire",
				maxAmmo = 100,
				time = 0,
				cool = 0.1,
				auto = true,
				fire = function(self, state)
					table.insert(state.playerFire, Bullet("bullet_rapid", self.posX + self.meta.width, self.posY + self.meta.height / 2, 1000))
				end
			}
		}
		
		self.weapon = weapon or self.inventory.cannon --WEAPON CLASS?
		self.weapon.ammo = self.weapon.maxAmmo
	end,

	update = function(self, dt, t)
		Entity.update(self, dt, t)

		if self.posX < 0 then
			self.posX = 0
			self.drawX = 0
			self.velocityX = 0
		elseif self.posX > 960 -self.meta.width then
			self.posX = 960 - self.meta.width
			self.drawX = 960 - self.meta.width
			self.velocityX = 0
		end

		if self.posY < 0 then
			self.posY = 0
			self.drawY = 0
			self.velocityY = 0
		elseif self.posY > 480 -self.meta.height then
			self.posY = 480 - self.meta.height
			self.drawY = 480 - self.meta.height
			self.velocityY = 0
		end
	end,

	draw = function(self)
		Entity.draw(self)
	end,

	damage = function(self, t, power, state)
		if t - (self.damageTime or 0) > 0.5 then
			self.damageTime = t
			self.life = self.life - (power or 1)
			table.insert(state.sprite, Shield(self))
			table.insert(state.sprite, Flash({255, 0, 0, 64}, 0.25))
		end
	end,

	fire = function(self, state, t)
		t = t or love.timer.getTime()
		if t - self.weapon.time > self.weapon.cool then
			self.weapon.time = t
			self.weapon.fire(self, state)
			self.weapon.ammo = self.weapon.ammo - (self.weapon.fireCost or 1)
			if self.weapon.ammo < 1 then
				self:changeWeapon("cannon")
			end
		end
	end,

	changeWeapon = function(self, weapon)
		self.weapon = self.inventory[weapon]
		self.weapon.ammo = self.weapon.maxAmmo
	end
}