--[[
	WORLD
	Gameplay state.
]]

return {
	reset = function(self, game, score)
		self.game = game

		self.score = 0
		if not score and love.filesystem.exists("high.scr") then
			score = self.game.file.load("high.scr")
			score = score[1]
		end
		self.highScore = score or 0
		self.lives = 5
		self.player = Player("temp", 100, 200)
		self.enemy = {}
		self.playerFire = {}
		self.enemyFire = {}
		self.entity = {}
		self.sprite = {}
		for x = 1, 5 do
			table.insert(self.enemy, ButterDroid())
		end
		self.scoreFont = cache.font("font/wendy.ttf:14")
		self.scoreText = {}

		self.bgImage = cache.image("spr/bg.png")
		self.bgBatch = love.graphics.newSpriteBatch(self.bgImage, 3)
		for x = 0, 1280, 640 do
			self.bgBatch:add(x, 0)
		end
		self.backgroundX = 0

		self.sharkSpawn = 1
		self.penguinSpawn = 1
		self.hugeSpawn = 1
		self.doubleSpawn = 1
		self.rockSpawn = 0
	end,

	update = function(self, dt, t)
		--handle bullets
		for x = # self.playerFire, 1, -1 do
			local bullet = self.playerFire[x]
			bullet:update(dt, t, self)
			if bullet.kill then
				table.remove(self.playerFire, x)
			end
		end

		for x = # self.enemyFire, 1, -1 do
			local bullet = self.enemyFire[x]
			bullet:update(dt, t, self)
			if bullet:collide(self.player.hitBox()) then
				bullet.kill = true
				self.player:damage(t, 1, self)
			end
			if bullet.kill then
				table.remove(self.enemyFire, x)
			end
		end

		--destory all enemies
		if self.crush then
			for x = # self.enemy, 1, -1 do
				local enemy = self.enemy[x]
				if isinstance(enemy, ButterDroid) then
					enemy:reposition()
				end
				if not isinstance(enemy, Powerup) and not isinstance(enemy, Crush) then
					enemy:death(self)
					table.remove(self.enemy, x)
					self.score = self.score + enemy.points
					table.insert(self.scoreText, {text = enemy.points, x = enemy.posX + 16, y = enemy.posY - 8, time = t})
				end
			end
			self.crush = nil
		end

		--handle enemies
		for x = # self.enemy, 1, -1  do
			local enemy = self.enemy[x]
			enemy:update(dt, t, self)

			--check player collision
			if enemy:collide(self.player.hitBox()) then
				enemy:death(self)
				if isinstance(enemy, ButterDroid) then
					enemy:reposition()
				else
					enemy.kill = true
				end
				if not isinstance(enemy, Powerup) and not isinstance(enemy, Crush) then
					self.player:damage(t, 1, self)
				end
			end
			--ghosted enemies can be hit but bullets pass through them
			if not enemy.ghost then
				--check bullet collision
				for k, bullet in ipairs(self.playerFire) do
					if enemy:collide(bullet.hitBox()) then
						enemy:damage(bullet.power)
						if enemy.life < 1 then
							self.score = self.score + enemy.points
							table.insert(self.scoreText, {text = enemy.points, x = bullet.posX + 16, y = bullet.posY - 8, time = t})
							enemy:death(self)

							if isinstance(enemy, ButterDroid) then
								enemy:reposition()
							else
								enemy.kill = true
							end
						end
						table.insert(self.sprite, Sprite("bullet_death", bullet.posX + 9, bullet.posY))
						table.remove(self.playerFire, k)
					end
				end
			end

			if enemy.kill then table.remove(self.enemy, x) end
		end
		
		--handle sprites
		for x = # self.sprite, 1, -1 do
			local sprite = self.sprite[x]
			sprite:update(dt, t)
			if sprite.kill then
				table.remove(self.sprite, x)
			end
		end

		--handle score text
		for x = # self.scoreText, 1, -1 do
			local text = self.scoreText[x]
			if t - text.time > 0.75 then
				table.remove(self.scoreText, x)
			else
				text.y = text.y - 24 * dt
			end
		end


		--check for gameover
		if self.player.life < 0 then
			self:reset(self.game, self.highScore)
		end
		--player input
		self.player.move.up = love.keyboard.isDown("w", "up")
		self.player.move.down = love.keyboard.isDown("s", "down")
		self.player.move.left = love.keyboard.isDown("a", "left")
		self.player.move.right = love.keyboard.isDown("d", "right")

		if self.player.weapon.auto and love.keyboard.isDown("k", "x") then
			self.player:fire(self, t)
		end

		self.player:update(dt, t)

		self.highScore = self.score > self.highScore and self.score or self.highScore

		if self.score / 5000 > # self.enemy - 4 then
			table.insert(self.enemy, ButterDroid())
		end

		if self.score / 3500 > self.sharkSpawn then
			self.sharkSpawn = self.sharkSpawn + 1
			table.insert(self.enemy, Jawz())
		end

		if self.score / 2000 > self.penguinSpawn then
			self.penguinSpawn = self.penguinSpawn + 1
			table.insert(self.enemy, Penguinator(self.player.posY))
		end

		if self.score / 25000 > self.hugeSpawn then
			self.hugeSpawn = self.hugeSpawn + 1
			for x = 1, 3 do
				table.insert(self.enemy, Penguinator(self.player.posY + 16 * (x - 1)))
				table.insert(self.enemy, Jawz())
			end
		end

		if self.score / 6000 > self.doubleSpawn then
			self.doubleSpawn = self.doubleSpawn + 1
			table.insert(self.enemy, Penguinator(self.player.posY + (self.player.velocityY > 0 and 40 or -40)))
		end

		if self.score / 9000 > self.rockSpawn then
			self.rockSpawn = self.rockSpawn + 1
			table.insert(self.enemy, Rock())
			if math.random() > 0.49 then
				table.insert(self.enemy, Rock(true))
			end
		end
		self.backgroundX = self.backgroundX - 600 * dt < -640 and self.backgroundX - 600 * dt + 640 or self.backgroundX - 600 * dt
	end,

	draw = function(self)
		love.graphics.setColor(color.white)
		love.graphics.draw(self.bgBatch, self.backgroundX, 0)
		for k, bullet in ipairs(self.playerFire) do
			bullet:draw()
		end
		for k, bullet in ipairs(self.enemyFire) do
			bullet:draw()
		end
		for k, enemy in ipairs(self.enemy) do
			enemy:draw()
		end

		self.player:draw()

		for k, sprite in ipairs(self.sprite) do
			sprite:draw()
		end
		love.graphics.setFont(self.scoreFont)
		for k, scoreText in ipairs(self.scoreText) do
			self.game:print(scoreText.text, scoreText.x, scoreText.y)
		end
		love.graphics.setFont(self.game.system.font)

		--hud
		local hudColor = {100, 150, 200, 255}
		self.game:print("SCORE", 60, 5, hudColor)
		self.game:print("LIVES", 5, 5, hudColor)
		self.game:print("HIGH SCORE", 300, 5, hudColor)
		self.game:print(self.score, 60, 17)
		self.game:print(self.player.life, 5, 17)
		self.game:print(self.highScore, 300, 17)

		self.game:print(self.player.weapon.name .. " x" .. self.player.weapon.ammo, 5, 29, {255, 150, 20, 255})

		-- love.graphics.print("# self.enemy " .. # self.enemy, 5, 465)
		if self.debug then
			self.game:print(# self.playerFire .. " bullets  " .. # self.enemy .. " enemies  " .. # self.sprite .. " sprites  " .. # self.scoreText .. " text  " .. getMemory() .. "Kb  " .. love.timer.getFPS() .. "fps", 5, 465)
		end
	end,

	keypressed = function(self, key, unicode)
		if key == "escape" then love.event.push("quit") end
		if key == "k" or key == "x" then
			self.player:fire(self)
			--table.insert(self.playerFire, Bullet("bullet", self.player.posX + self.player.meta.width, self.player.posY + self.player.meta.height / 2))
		elseif key == "f1" then
			self.debug = not self.debug
		end
	end,

	quit = function(self)
		self.game.file.save({_fileName = "high.scr", self.highScore})
	end
}